@@ -179,15 +179,23 @@ class DbOne(Speaker):
             '_rollback',
             '_commit',
             '_gquery',
-            '_qquery'
+            '_qquery',
+            '_mqquery',
+            '_mgquery'
             ]
 
-    # Iface methods
-    # _connect()  
-    # _rollback()
-    # _commit()
-    # _get_query(qry, filter)
-    # _quiet_query(qry, filter)
+    _exported_methods= [
+            'lastrowid',
+            'oc_query',
+            'ov_query',
+            'mr_query',
+            'or_query',
+            'mq_query',
+            'mg_query',
+            'beginTrans',
+            'commit',
+            'rollback',
+            ]
 
     @staticmethod
     def _import_iface():
@@ -197,6 +205,23 @@ class DbOne(Speaker):
         if hasattr(iface, '_mogrify'):
             DbOne.mogrify= staticmethod(iface._mogrify)
 
+    @staticmethod
+    def export_iface(where):
+
+        if not DbOne._db_initialized:
+            err= ERR_COL('!!!No Session Available!!!')
+            Speaker.log_int_raise("%s You are using the Db Pool, you "
+                               "have disabled the persistent connection, "
+                               "but no session was used to initialize this "
+                               "Mother class", err)
+
+        for attr in DbOne._exported_methods:
+            setattr(where, attr, getattr(DbOne, attr))
+
+        mog= getattr(DbOne, 'mogrify', None)
+        if mog:
+            setattr(where, 'mogrify', mog)
+
     @staticmethod
     def beginTrans():
         DbOne.trans_level+= 1
@@ -244,26 +269,7 @@ class DbOne(Speaker):
         return DbOne._iface_instance._lastrowid()
 
     @staticmethod
-    def _do_query(s, filter= None, result= True):
-
-        execattr= result and DbOne._gquery or DbOne._qquery
-
-        if not filter:
-            d= {}
-        elif isinstance(filter, MoFilter):
-            s, d= filter.return_ftrqry(s)
-        elif isinstance(filter, dict):
-            d= filter
-        else:
-            Speaker.log_int_raise('Invalid Filter Type: %s', 
-                    ERR_COL(type(filter)))
-
-        # logging info...
-        mogrify= getattr(DbOne._iface_instance, '_mogrify', None)
-        if mogrify is not None:
-            Speaker.log_info("QSQL- %s", mogrify(s, d))
-        else:
-            Speaker.log_info("QSQL- %s, Filter= %s" % (s, d))
+    def _safe_execute(execattr, s, d):
 
         try:
             return execattr(s, d)
@@ -291,6 +297,29 @@ class DbOne(Speaker):
             Speaker.log_raise('%s queries: %s.', 
                     ERR_COL('Rollbacked'), ERR_COL(e), QueryError)
 
+    @staticmethod
+    def _do_query(s, filter= None, result= True):
+
+        execattr= result and DbOne._gquery or DbOne._qquery
+
+        if not filter:
+            d= {}
+        elif isinstance(filter, MoFilter):
+            s, d= filter.return_ftrqry(s)
+        elif isinstance(filter, dict):
+            d= filter
+        else:
+            Speaker.log_int_raise('Invalid Filter Type: %s', 
+                    ERR_COL(type(filter)))
+
+        # logging info...
+        mogrify= getattr(DbOne._iface_instance, '_mogrify', None)
+        if mogrify is not None:
+            Speaker.log_info("QSQL- %s", mogrify(s, d))
+        else:
+            Speaker.log_info("QSQL- %s, Filter= %s" % (s, d))
+
+        return DbOne._safe_execute(execattr, s, d)
 
     @staticmethod
     def _get_query(s, filter= None):
@@ -344,6 +373,30 @@ class DbOne(Speaker):
 
         return res[0]
 
+    @staticmethod
+    def mq_query(s, l):
+        """ Multiple Quiet Query: execute a multiple, quiet query."""
+
+        Speaker.log_debug("Executing Massive Query...")
+
+        DbOne._safe_execute(DbOne._mqquery, s, l)
+
+        if not DbOne.trans_level:
+            DbOne._commit()
+
+    @staticmethod
+    def mg_query(s, l):
+        """ Multiple Get Query: execute a multiple, get query."""
+
+        Speaker.log_debug("Executing Massive Query...")
+
+        res= DbOne._safe_execute(DbOne._mgquery, s, l)
+
+        if not DbOne.trans_level:
+            DbOne._commit()
+
+        return res
+
 class DbFly(Speaker):
 
     # Iface methods
@@ -362,15 +415,19 @@ class DbFly(Speaker):
             '_commit',
             '_gquery',
             '_qquery',
+            '_mqquery',
+            '_mgquery',
             '_close'
             ]
 
-    exported_methods= [
+    _exported_methods= [
             'lastrowid',
             'oc_query',
             'ov_query',
             'mr_query',
             'or_query',
+            'mq_query',
+            'mg_query',
             'beginTrans',
             'commit',
             'rollback',
@@ -394,9 +451,9 @@ class DbFly(Speaker):
         if hasattr(self._iface_instance, '_mogrify'):
             self.mogrify= self._iface_instance._mogrify
 
-    def _export_iface(self, where):
+    def export_iface(self, where):
 
-        for attr in self.exported_methods:
+        for attr in self._exported_methods:
             setattr(where, attr, getattr(self, attr))
 
         mog= getattr(self, 'mogrify', None)
@@ -449,25 +506,7 @@ class DbFly(Speaker):
 
         MotherPool.backHome(self)
 
-    def _do_query(self, s, filter= None, result= True):
-
-        execattr= result and self._gquery or self._qquery
-
-        if not filter:
-            d= {}
-        elif isinstance(filter, MoFilter):
-            s, d= filter.return_ftrqry(s)
-        elif isinstance(filter, dict):
-            d= filter
-        else:
-            self.log_int_raise('Invalid Filter Type: %s', 
-                    ERR_COL(type(filter)))
-
-        mogrify= getattr(self._iface_instance, '_mogrify', None)
-        if mogrify is not None:
-            self.log_info("%s: QSQL- %s", INF_COL(self.session_name), mogrify(s, d))
-        else:
-            self.log_info("%s: QSQL- %s, Filter= %s" , INF_COL(self.session_name), s, d)
+    def _safe_execute(self, execattr, s, d):
 
         try:
             res= execattr(s, d)
@@ -495,6 +534,27 @@ class DbFly(Speaker):
             self.log_raise('%s queries: %s.', ERR_COL('Rollbacked'), 
                     ERR_COL(e), QueryError)
 
+    def _do_query(self, s, filter= None, result= True):
+
+        execattr= result and self._gquery or self._qquery
+
+        if not filter:
+            d= {}
+        elif isinstance(filter, MoFilter):
+            s, d= filter.return_ftrqry(s)
+        elif isinstance(filter, dict):
+            d= filter
+        else:
+            self.log_int_raise('Invalid Filter Type: %s', 
+                    ERR_COL(type(filter)))
+
+        mogrify= getattr(self._iface_instance, '_mogrify', None)
+        if mogrify is not None:
+            self.log_info("%s: QSQL- %s", INF_COL(self.session_name), mogrify(s, d))
+        else:
+            self.log_info("%s: QSQL- %s, Filter= %s" , INF_COL(self.session_name), s, d)
+
+        return self._safe_execute(execattr, s, d)
 
     def _get_query(self, s, filter= None):
 
@@ -527,7 +587,7 @@ class DbFly(Speaker):
     def ov_query(self, s, filter= None):
         """ One Value Query: returns a unique value."""
         
-        res= DbOne.or_query(s, filter)
+        res= self.or_query(s, filter)
 
         res= res.values()
         if len(res)<> 1:
@@ -537,6 +597,26 @@ class DbFly(Speaker):
 
         return res[0]
 
+    def mq_query(self, s, l):
+        """ Multiple Quiet Query: execute a multiple quiet query."""
+
+        Speaker.log_debug("%s: Executing Massive Query...", 
+                        INF_COL(self.session_name))
+
+        res= self._safe_execute(self._mqquery, s, l)
+        # len(l) - 1: because a query is added by safe_execute
+        self._queries_n+= len(l) - 1
+
+    def mg_query(self, s, l):
+        """ Multiple Get Query: execute a multiple get query."""
+
+        Speaker.log_debug("%s: Executing Massive Query...", 
+                        INF_COL(self.session_name))
+
+        res= self._safe_execute(self._mgquery, s, l)
+        # len(l) - 1: because a query is added by safe_execute
+        self._queries_n+= len(l) - 1
+        return res
 
 class MotherPool:
 
@@ -804,6 +884,9 @@ def init_abdbda(conf, forced= {}):
         Speaker.log_warning('AbDbDa already initialized.')
         return
 
+    # Take trace of possible errors: 
+    # When Speaker will be initialized, it 
+    # will be possible to log them
     err= None
     if not isinstance(conf, dict):
         import speaker
