@@ -11,6 +11,8 @@ Where the main Mother Classes and methods are defined:
  * DbMother
  * MotherManager
  * MotherBox
+ * MotherFusion
+ * MotherMany
  * MotherSession
  * MotherPoolStatus
  * MotherPoolStratus
@@ -18,6 +20,7 @@ Where the main Mother Classes and methods are defined:
             
 from commons import ERR_COL, INF_COL, OKI_COL
 from abdbda import DbOne, MoFilter
+from speaker import Speaker
 
 #
 ##
@@ -76,7 +79,6 @@ def init_mother(cfile, fnaming= None):
     """
 
     if not hasattr(DbMother, '_mo_initialized'):
-        from speaker import Speaker
         Speaker.log_warning("Mother already initialized.")
         return
 
@@ -87,22 +89,23 @@ def init_mother(cfile, fnaming= None):
     names_dict.update(abdbda.__dict__)
     execfile(cfile, names_dict, d)
 
-    from abdbda import DB_ENGINE_PGRES
+    DB_ENGINE_PGRES= abdbda.DB_ENGINE_PGRES
     db_engine= d.get('DB_ENGINE', DB_ENGINE_PGRES)
     if db_engine == DB_ENGINE_PGRES:
         DbMother._sqlInsert= DbMother._sqlPostgresInsert
-        DbMother._mo_arg_format= '%%(%s)s'
+        _DbMap._mo_arg_format= '%%(%s)s'
 
         use_oids= d.get('MOTHER_OIDS', True)
         if not use_oids:
             DbMother._mo_pg_oids= False
             abdbda.DbFly.exported_methods.remove('lastrowid')
+            abdbda.DbOne.exported_methods.remove('lastrowid')
         else:
             DbMother._mo_pg_oids= True
 
     else:
         DbMother._sqlInsert= DbMother._sqlSqliteInsert
-        DbMother._mo_arg_format= ':%s'
+        _DbMap._mo_arg_format= ':%s'
 
     del DB_ENGINE_PGRES
 
@@ -119,12 +122,12 @@ def init_mother(cfile, fnaming= None):
 
     # Design Mother
     if fnaming:
-        from speaker import Speaker, RED
         Speaker.log_insane("Testing fnaming function...")
         try:
             res_str= fnaming('foo')
             assert isinstance(res_str, str)
         except:
+            from speaker import RED
             Speaker.log_warning(
                     "Testing %s. Default fnaming function "
                     "will be used.", RED('Failed!'))
@@ -193,16 +196,26 @@ def MotherPoolStratus():
     Connections Employed    ~ %s 
     """ % MotherPoolStatus()
 
-class _DbMap(DbOne):
+class _DbMap(Speaker):
     _map_file=       None
     _map_fields=     None
     _map_pkeys=      None
     _map_children=   None
     _map_rels=       None
 
+    # Mapping actions
+    @staticmethod
+    def _flag_actions(obj, flag):
+        if flag == MO_UP: return obj.update
+        if flag == MO_DEL: return obj.delete
+        if flag == MO_NOA: return _do_nothing
+        if flag == MO_SAVE: return obj.insert
+        if flag == MO_LOAD: return obj.load
+        if flag == MO_ULOAD: return obj.uload 
+        obj.log_int_raise("Invalid flag %s", ERR_COL(flag))
+
     @staticmethod
     def _load_map(map_file):
-        from speaker import Speaker
         from os import path
         if not path.isfile(map_file):
             Speaker.log_int_raise("Mother Map %s does not exist.", ERR_COL(map_file)) 
@@ -255,7 +268,6 @@ class _DbMap(DbOne):
         except:
             pass
 
-        from speaker import Speaker
         Speaker.log_insane(\
             "Warning: direct relation for tables %s "
             "is not defined: searching subset...",
@@ -308,12 +320,48 @@ class _DbMap(DbOne):
 
         s+=" WHERE "
         s+=_A(
-            ["%s.%s = %s " % (r_tbl, v, DbMother._mo_arg_format % k)
+            ["%s.%s = %s " % (r_tbl, v, _DbMap._mo_arg_format % k)
                 for k, v in r_key.iteritems()])
 
         return s
 
-    def _sqlJoinParent(self, builder, j_builder, outer=False):
+    def _arg_format(self, k):
+        f= _DbMap._mo_arg_format
+        return f % k
+
+    def _unvaluedPKeys(self):
+        """ _unvaluedPKeys() --> set
+
+        Returns the set of pkeys not valu'ed.
+        NOTE: a pkeys is considere NOT VALUED if his value is 
+        DEFAULT or NULL.
+        """
+
+        s=self._store
+        return set([                \
+            k for k in self.pkeys   \
+            if not s.has_key(k)     \
+            or s[k] in self._mo_false_values])
+
+    def _invalidFields(self,fields):
+        """ _invalidFields(fields) --> set
+
+        field in fields are returned if they are not present
+        in self.fields.
+        """
+
+        if type(fields) is not set:
+            fields= set(fields)
+        return fields - self.fields
+
+    def _fieldsMissing(self):
+        """ _fieldsMissing() --> set
+
+        Returns the list of table fields not valu'ed yet.
+        """
+        return self.fields - set(self._store.keys())
+
+    def _sqlJoinParent(self, builder, j_builder, outer= False):
         """ _sqlJoinParent(self, builder, j_builder [, outer= False]) --> (string, string)
 
         Returns a filter and the table where filter has to be applied.
@@ -453,17 +501,6 @@ class DbMother(_DbMap):
             MO_ULOAD: 'MO_ULOAD'
             }
         
-    # Mapping actions
-    @staticmethod
-    def _flag_actions(obj, flag):
-        if flag == MO_UP: return obj.update
-        if flag == MO_DEL: return obj.delete
-        if flag == MO_NOA: return _do_nothing
-        if flag == MO_SAVE: return obj.insert
-        if flag == MO_LOAD: return obj.load
-        if flag == MO_ULOAD: return obj.uload 
-        obj.log_int_raise("Invalid flag %s", ERR_COL(flag))
-
     @staticmethod
     def _trigger_actions(obj, flag):
         if flag == MO_UP: return obj._update
@@ -490,19 +527,12 @@ class DbMother(_DbMap):
         tbl= self.table_name
 
         # Session() ?
-        if session:
-            session._export_iface(self)
-
-        elif not DbOne._db_initialized:
-            err= ERR_COL('!!!No Session Available!!!')
-            self.log_int_raise("%s You are using the Db Pool, you "
-                               "have disabled the persistent connection, "
-                               "but no session was used to initialize this "
-                               "Mother class (table= %s)", err, ERR_COL(tbl))
+        iface= session or DbOne
+        iface.export_iface(self)
         self.session= session
 
-        if hasattr(self._iface_instance, '_mogrify'):
-            self.mogrify= self._iface_instance._mogrify
+        #if hasattr(self._iface_instance, '_mogrify'):
+            #self.mogrify= self._iface_instance._mogrify
 
         # self.fields is a list with TABLE fields.
         # self.pkeys is a list with primary keys for the TABLE.
@@ -567,7 +597,7 @@ class DbMother(_DbMap):
     def setParanoia(self):
         setattr(self, 'paranoid', 1)
 
-    def unserParanoia(self):
+    def unsetParanoia(self):
         setattr(self, 'paranoid', 0)
         
 
@@ -579,42 +609,6 @@ class DbMother(_DbMap):
     def _triggers_are_initialized(self):
         return hasattr(self, 'triggers_map')
 
-    def _arg_format(self, k):
-        f= DbMother._mo_arg_format
-        return f % k
-
-    def _unvaluedPKeys(self):
-        """ _unvaluedPKeys() --> set
-
-        Returns the set of pkeys not valu'ed.
-        NOTE: a pkeys is considere NOT VALUED if his value is 
-        DEFAULT or NULL.
-        """
-
-        s=self._store
-        return set([                \
-            k for k in self.pkeys   \
-            if not s.has_key(k)     \
-            or s[k] in self._mo_false_values])
-
-    def _invalidFields(self,fields):
-        """ _invalidFields(fields) --> set
-
-        field in fields are returned if they are not present
-        in self.fields.
-        """
-
-        if type(fields) is not set:
-            fields=set(fields)
-        return fields - self.fields
-
-    def _fieldsMissing(self):
-        """ _fieldsMissing() --> set
-
-        Returns the list of table fields not valu'ed yet.
-        """
-        return self.fields - set(self._store.keys())
-
     def __str__(self):
         """ __str__() --> string 
         """
@@ -1312,6 +1306,14 @@ class MotherManager:
                                 c_dict,
                                 MO_DEL,
                               )
+    def handleManyChildren(self, cbuilder, flag, c_dicts, fields= None):
+
+        child_table= cbuilder.table_name
+        d= self.getChildDeps(child_table)
+        d= self._getDepDict(d)
+        for c in c_dicts:
+            c.update(d)
+        return MotherMany(cbuilder, c_dicts, flag, self.session, fields= fields)
 
     def deleteChildren(self, cbuilder, filter= None):
         """ deleteChildren(cbuilder [,filter=None]) --> None
@@ -1616,6 +1618,49 @@ class MotherManager:
         if flag_obj: return r_obj
         else: return r_obj.getFields()
 
+    def initManyManager(self, children):
+
+        self.log_info("The initManyManager() methods are not tested deeply: if You "\
+                      "encounter bugs, don't panic: please, signal them.")
+
+        def handle_children(builder, flag):
+            def fly_handle_children(dlist, fields= None):
+                return self.handleManyChildren(builder, flag, dlist, fields)
+            return fly_handle_children
+
+        camel= DbMother.MotherNaming
+
+        map_names= {
+                MO_LOAD: 'loadMany', 
+                MO_SAVE: 'insertMany', 
+                MO_UP: 'updateMany', 
+                MO_DEL: 'deleteMany'
+                }
+
+        for c in children:
+          ctbl= c.table_name
+          for flag, name in map_names.iteritems():
+
+            attr_name= camel(ctbl, name)
+            if not hasattr(self, attr_name): 
+
+              attr= handle_children(c, flag)
+              if flag in [MO_UP, MO_LOAD]:
+                attr.__doc__=\
+                  " %s(dlist [, fields= None) --> MotherMany instance\n\n"        \
+                  "handle many children. dlist is a list of dicts, "              \
+                  "fields is a list of strings.\n"                                \
+                  "Don't specify the foreign key: it's assigned automatically.\n" \
+                      % attr_name
+              else:
+                attr.__doc__=\
+                  " %s(dlist) --> MotherMany instance\n\n"                        \
+                  "handle many children. dlist is a list of dicts.\n"             \
+                  "Don't specify the foreign key: it's assigned automatically.\n" \
+                      % attr_name
+
+              setattr(self, attr_name, attr)
+
     def initChildManager(self, children):
         """initChildManager(self,children) --> None
         
@@ -1793,7 +1838,7 @@ class MotherManager:
         def rel_params(builder):
             def fly_rel_params(d, fields= None, flag_obj= False):
                 if not isinstance(d, DbMother):
-                    d= builder(d)
+                    d= builder(d, session= self.session)
                 return self.relParams([d], fields, flag_obj)
             return fly_rel_params
 
@@ -1875,10 +1920,10 @@ class MotherBox(DbOne):
                     fields= None, order= None, session= None, 
                     distinct= False, notriggers= False):
 
-        if session:
-            session._export_iface(self)
-
+        iface= session or DbOne
+        iface.export_iface(self)
         self.session= session
+
         self.builder= builder
         self.momma= builder(session= session)
         self.notriggers= notriggers
@@ -2011,7 +2056,7 @@ class MotherBox(DbOne):
             newd[newk] = v
             if v== SQL_DEFAULT:
                 res.append('%s= DEFAULT' % k)
-            if v== SQL_DEFAULT:
+            if v== SQL_NULL:
                 res.append('%s= NULL' % k)
             else:
                 res.append('%s= %s' % (k, self.momma._arg_format(newk)))
@@ -2091,13 +2136,15 @@ class MotherFusion(_DbMap):
 
     def __init__(self, builderA, builderB, filter= None, fields= None,
                     order= None, session= None, distinct= False, 
-                    rtbl= None, params= False, jfilter= None):
+                    rtbl= None, params= False, jfilter= None, side= ""):
 
         self.direct= None
+        self.side= side
 
         builderA, builderB, fields= self.swap(builderA, builderB, fields)
 
-        if session: session._export_iface(self)
+        iface= session or DbOne
+        iface.export_iface(self)
         self.session= session
 
         if filter:
@@ -2109,7 +2156,6 @@ class MotherFusion(_DbMap):
 
         self.jfilter= jfilter
 
-
         if order:
             self.filter.add_post('ORDER BY %s' % _J(order))
 
@@ -2173,29 +2219,24 @@ class MotherFusion(_DbMap):
         # recursion
         sw= self._selectWhat
 
-        # no params:
-        if len(fields) == 2:
-            a= sw(fields[0], self.tblA)
-            b= sw(fields[1], self.tblB)
+        fa= fields[0]
+        fb= fields[1]
 
-            if a and b:
-                return _J([a, b])
-            return a or b
+        a= fa and sw(fa, self.tblA) or '%s.*' % self.tblA
+        b= fb and sw(fb, self.tblB) or '%s.*' % self.tblB
 
-        a= sw(fields[0], self.tblA)
-        b= sw(fields[1], self.tblB)
-        r= sw(fields[2], self.rtbl)
+        if len(fields) == 3:
+            fr= fields[2]
+            r= fr and sw(fr, self.rtbl) or '%s.*' % self.rtbl
+            
+            return _J([a, b, r])
 
-        # if you provided three dict, it's possible
-        # to assume that params are specified.
-        if a and b:
-            return _J(a, b, r)
+        return _J([a, b])
 
-        return _J(a or b, r)
 
     def joinBuilders(self):
 
-	self.log_insane('MotherFusion: join relation')
+        self.log_insane('MotherFusion: join relation')
         # useful vars
         ba= self.builderA
         bb= self.builderB
@@ -2236,7 +2277,8 @@ class MotherFusion(_DbMap):
 
         what= self._selectWhat(self.fields)
         distinct= self.distinct and 'DISTINCT' or ''
-        qry= 'SELECT %(distinct)s %(what)s FROM %(ta)s' % locals()
+        side= self.side
+        qry= 'SELECT %(distinct)s %(what)s FROM %(ta)s %(side)s' % locals()
 
         self._store= self.mr_query(qry, ftr)
 
@@ -2247,14 +2289,16 @@ class MotherFusion(_DbMap):
         bb= self.builderB
         ta= ba.table_name
         tb= bb.table_name
-	self.log_insane('MotherFusion: direct join: %s father, %s child', 
-			INF_COL(ta), INF_COL(tb))
+        self.log_insane('MotherFusion: direct join: %s father, '
+                        '%s child', INF_COL(ta), INF_COL(tb))
 
         what= self._selectWhat(self.fields)
         jfilter, joining_table= self._sqlJoinParent(ba, bb)
 
         distinct= self.distinct and 'DISTINCT' or ''
-        qry= 'SELECT %(distinct)s %(what)s FROM %(ta)s %(jfilter)s' % locals()
+        side= self.side
+        qry= 'SELECT %(distinct)s %(what)s FROM %(ta)s ' \
+                '%(side)s %(jfilter)s' % locals()
         self._store= self.mr_query(qry, self.filter)
 
     def getRecords(self, flag_obj= False):
@@ -2299,3 +2343,150 @@ class MotherFusion(_DbMap):
             for a, r, b in zip(dla, dlr, dlb)]
 
 
+class MotherMany(_DbMap):
+
+    def __init__(self, builder, store= None, flag= MO_NOA, 
+                    session= None, fields= None):
+
+        self.log_info("The MotherMany class is not tested deeply: if You "\
+                      "encounter bugs, don't panic: please, signal them.")
+        if isinstance(builder, str):
+            self.builder= None
+            self.table_name= builder
+        else:
+            self.builder= builder
+            self.table_name= builder.table_name
+
+        iface= session or DbOne
+        iface.export_iface(self)
+        self.session= session
+
+        if isinstance(store, dict):
+            store= [store]
+
+        self.store= store
+        self._act_fields= fields
+
+        f= self._flag_actions(self, flag)
+        f()
+
+    def __len__(self):
+
+        return len(self._records)
+
+    def addRows(self, rows):
+
+        try:
+            self.store.extend(rows)
+
+        except:
+            if not isinstance(rows, dict):
+                self.log_int_raise("Invalid rows type: %s", ERR_COL(rows))
+            self.store.append(rows)
+
+        self._records= self.store
+
+    def insert(self):
+
+        s= self.store
+        if not s:
+            self.log_info("MotherMany.insert(): empty store. Skipping")
+            return 
+
+        af= self._arg_format
+
+        f= s[0].keys()
+        fields= _J(f)
+        table= self.table_name
+        values= _J([af(k) for k in f])
+
+        qry= 'INSERT INTO %(table)s (%(fields)s) VALUES (%(values)s)' % locals()
+        self.log_info("MotherMany: Inserting on %s %s row(s) (template= `%s`)...", 
+                        INF_COL(table), INF_COL(len(s)), INF_COL(qry))
+        self.mq_query(qry, s)
+
+        self._records= self.store
+
+
+    def delete(self):
+
+        s= self.store
+        if not s:
+            self.log_info("MotherMany.delete(): empty store. Skipping")
+            return 
+
+        af= self._arg_format
+
+        fields= s[0].keys()
+        table= self.table_name
+        filter= _A([ "%s = %s" % (j, af(j)) for j in fields])
+
+        qry= 'DELETE FROM %(table)s WHERE %(filter)s' % locals()
+
+        self.log_info("MotherMany: Deleting on %s with %s filter(s) "
+                      "(template= `%s`)...", table, INF_COL(len(s)), INF_COL(qry))
+
+        self.mq_query(qry, s)
+        self._records= self.store
+
+    def update(self):
+
+        s= self.store
+        if not s:
+            self.log_info("MotherMany.update(): empty store. Skipping")
+            return 
+
+        table= self.table_name
+        fields= s[0].keys()
+        act= self._act_fields
+        if not act:
+            act= [f for f in fields if f not in self._table_pkeys(table)]
+        if not act or len(act) == len(fields):
+            self.log_int_raise('MotherMany.update(): cannot understand what '
+                                'has to be updated.')
+
+        af= self._arg_format
+
+        upup= _J(['%s= %s' % (f, af(f)) for f in act])
+        filter= _A(['%s= %s' % (f, af(f)) for f in fields if f not in act])
+
+        qry= 'UPDATE %(table)s SET %(upup)s WHERE %(filter)s' % locals()
+
+        self.log_info("MotherMany: Updating %s with %s filter(s) "
+                      "(template= `%s`)...", table, INF_COL(len(s)), INF_COL(qry))
+
+        self.mq_query(qry, s)
+        self._records= self.store
+
+    def load(self):
+
+        s= self.store
+        if not s:
+            self.log_info("MotherMany.select(): empty store. Skipping")
+            return 
+
+        af= self._arg_format
+        fields= s[0].keys()
+        table= self.table_name
+        act= self._act_fields
+        what= act and _J(act) or '*'
+        filter= _A([ "%s = %s" % (j, af(j)) for j in fields])
+
+        qry= 'SELECT %(what)s FROM %(table)s WHERE %(filter)s' % locals()
+
+        self.log_info("MotherMany: Loading on %s with %s filter(s) "
+                      "(template= `%s`)...", table, INF_COL(len(s)), INF_COL(qry))
+
+        self._records= self.mg_query(qry, s)
+
+    def getRecords(self, flag_obj= False):
+        """ getRecords(self, flag_obj= False) -> list(dict) or list(Mothers)
+
+        The return value depends on flag_obj.
+        """
+        if flag_obj:
+            b= self.builder or getMotherBuilder(self.table_name)
+            session= self.session
+            return [b(d, MO_NOA, session) for d in self._records]
+        else:
+            return self._records
