@@ -1,5 +1,4 @@
 import psycopg2
-from psycopg2.extras import DictCursor as _cursor_factory
 from mother.speaker import Speaker
 from mother.commons import ERR_COL
 
@@ -37,11 +36,10 @@ class MotherPostgres:
     def _connect(self):
 
         Speaker.log_insane('Initializing postgres connection...')
-        from psycopg2.extras import DictCursor as _cf
         try:
             s= _PostgresInfo._connect_str()
             self.connection= psycopg2.connect(s)
-            self.cursor= self.connection.cursor(cursor_factory= _cf)
+            self.cursor= self.connection.cursor()
         except Exception, ss:
             Speaker.log_raise('Unable to establish a connection '
                     'with the database: %s', ERR_COL(ss), 
@@ -56,6 +54,21 @@ class MotherPostgres:
     def _close(self):
         self.connection.close()
 
+    def _extract(self):
+
+        c= self.cursor
+        cres= c.fetchall()
+        desc= c.description
+        res= []
+
+        for rec in cres:
+            drec= {}
+            for n, field in enumerate(rec):
+                drec[desc[n][0]]= field
+            res.append(drec)
+
+        return res
+
     def _lastrowid(self):
         return self.cursor.lastrowid
 
@@ -66,28 +79,39 @@ class MotherPostgres:
         except psycopg2.OperationalError:
             Speaker.log_raise('Connection is broken...', BrokenConnection)
         except Exception, ss:
-            Speaker.log_raise(ERR_COL(ss), QueryError)
+            Speaker.log_raise('%s', ERR_COL(ss), QueryError)
 
+    def _executemany(self, q, l):
+
+        try:
+            self.cursor.executemany(q, l)
+        except psycopg2.OperationalError:
+            Speaker.log_raise('Connection is broken...', BrokenConnection)
+        except Exception, ss:
+            Speaker.log_raise('%s', ERR_COL(ss), QueryError)
 
     def _gquery(self, q, d):
+
         self._execute(q, d)
-        c= self.cursor
-        res= c.fetchall()
 
-        # we need to conver dictRow to dict
-        lres= []
-        for r in res:
-            d= {}
-            d.update(r)
-            lres.append(d)
+        return self._extract()
 
-        return lres
-    
     def _qquery(self, q, d):
 
         self._execute(q, d)
 
+    def _mqquery(self, q, l):
+
+        self._executemany(q, l)
+
+    def _mgquery(self, q, l):
+
+        self._executemany(q, l)
+
+        return self._extract()
+
     def _mogrify(self, q, d):
+
         return self.cursor.mogrify(q, d)
 
     def get_tables(self):
