@@ -42,6 +42,25 @@ class MotherSqlite:
     def _close(self):
         self.connection= None
 
+    def _extract(self):
+        c= self.cursor
+        # cannot understand why, if no result is fetched,
+        # description is not available....
+        try:
+            desc= c.getdescription()
+        except apsw.ExecutionCompleteError:
+            # forcing to return no results
+            return []
+
+        res= []
+        for rec in c:
+            drec= {}
+            for n, field in enumerate(rec):
+                drec[desc[n][0]]= field
+            res.append(drec)
+
+        return res
+
     def _lastrowid(self):
         return self.connection.last_insert_rowid()
 
@@ -50,25 +69,33 @@ class MotherSqlite:
         try:
             self.cursor.execute(q, d)
         except Exception, ss:
-            Speaker.log_raise(ERR_COL(ss), QueryError)
+            Speaker.log_raise('%s', ERR_COL(ss), QueryError)
+
+    def _executemany(self, q, l):
+
+        try:
+            self.cursor.executemany(q, l)
+        except Exception, ss:
+            Speaker.log_raise('%s', ERR_COL(ss), QueryError)
 
     def _gquery(self, q, d):
-        self._execute(q, d)
-        c= self.cursor
-        res= []
-        for rec in c:
-            drec= {}
-            rec_descr= c.getdescription()
-            for n, field in enumerate(rec):
-                drec[rec_descr[n][0]]= field
-            res.append(drec)
 
-        return res
+        self._execute(q, d)
+        return self._extract()
     
     def _qquery(self, q, d):
 
         self._execute(q, d)
 
+    def _mqquery(self, q, l):
+
+        self._executemany(q, l)
+
+    def _mgquery(self, q, l):
+
+        self._executemany(q, l)
+        return self._extract()
+
     def get_tables(self):
         qry= ("SELECT tbl_name FROM SQLITE_MASTER WHERE type='table'")
         res= self._gquery(qry, {})
